from PIL import Image, ImageFile
import os
import cv2
import numpy as np
import re

# تنظیم برای بارگذاری تصاویر ناقص
ImageFile.LOAD_TRUNCATED_IMAGES = True

images_folder = 'images/'
output_folder = 'output_images/'
os.makedirs(output_folder, exist_ok=True)

def add_white_background(img):
    if img.mode in ('RGBA', 'LA') or (img.mode == 'P' and 'transparency' in img.info):
        background = Image.new("RGB", img.size, (255, 255, 255))
        background.paste(img.convert('RGBA'), (0, 0), img.convert('RGBA'))
        return background
    else:
        return img.convert("RGB")

def extract_largest_subject(image):
    gray = cv2.cvtColor(image, cv2.COLOR_BGR2GRAY)
    _, thresh = cv2.threshold(gray, 240, 255, cv2.THRESH_BINARY_INV)
    kernel = cv2.getStructuringElement(cv2.MORPH_RECT, (5, 5))
    morphed = cv2.morphologyEx(thresh, cv2.MORPH_CLOSE, kernel, iterations=2)
    contours, _ = cv2.findContours(morphed, cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_SIMPLE)

    if contours:
        largest_contour = max(contours, key=cv2.contourArea)
        x, y, w, h = cv2.boundingRect(largest_contour)
        return x, y, w, h
    return None

def process_image(image_path, target_size=None):
    try:
        image_pil = Image.open(image_path)
        image_pil = add_white_background(image_pil)
        image = cv2.cvtColor(np.array(image_pil), cv2.COLOR_RGB2BGR)

        subject_box = extract_largest_subject(image)

        if subject_box:
            x, y, w, h = subject_box
            padding_x = int(w * 0.03)
            padding_y = int(h * 0.03)

            start_x = max(x - padding_x, 0)
            end_x = min(x + w + padding_x, image.shape[1])
            start_y = max(y - padding_y, 0)
            end_y = min(y + h + padding_y, image.shape[0])

            cropped_image = image[start_y:end_y, start_x:end_x]

            background_color = (255, 255, 255)
            cropped_image_pil = Image.fromarray(cv2.cvtColor(cropped_image, cv2.COLOR_BGR2RGB))

            if target_size:
                ratio = min((target_size[0] * 0.9) / cropped_image_pil.width, (target_size[1] * 0.9) / cropped_image_pil.height)
                new_size = (int(cropped_image_pil.width * ratio), int(cropped_image_pil.height * ratio))
                resized_image = cropped_image_pil.resize(new_size, Image.LANCZOS)

                final_image = Image.new("RGB", target_size, background_color)
                position = ((target_size[0] - new_size[0]) // 2, (target_size[1] - new_size[1]) // 2)
                final_image.paste(resized_image, position)
            else:
                target_size = (800, 800)
                ratio = min((target_size[0] * 0.8) / cropped_image_pil.width, (target_size[1] * 0.8) / cropped_image_pil.height)
                new_size = (int(cropped_image_pil.width * ratio), int(cropped_image_pil.height * ratio))
                resized_image = cropped_image_pil.resize(new_size, Image.LANCZOS)

                final_image = Image.new("RGB", target_size, background_color)
                position = ((target_size[0] - new_size[0]) // 2, (target_size[1] - new_size[1]) // 2)
                final_image.paste(resized_image, position)

            final_image.save(image_path, 'JPEG', quality=95)
            print(f"Processed and saved {image_path}")
        else:
            print(f"No significant subject found in {image_path}")
    except OSError as e:
        print(f"Error processing {image_path}: {e}")

def get_size_from_filename(filename):
    # پیدا کردن الگوهای عرض x ارتفاع در نام فایل
    matches = re.findall(r'(\d+)x(\d+)', filename)
    if matches:
        # بررسی اینکه آیا آخرین جفت عدد در ۱۰ کاراکتر انتهایی است
        last_match = matches[-1]
        start_index = filename.rfind(last_match[0] + 'x' + last_match[1])
        if start_index >= len(filename) - 10:
            return int(last_match[0]), int(last_match[1])
    return None

def process_images_in_folder(folder):
    for root, dirs, files in os.walk(folder):
        for image_file in files:
            if image_file.lower().endswith(('.png', '.jpg', '.jpeg')):
                image_path = os.path.join(root, image_file)
                target_size = get_size_from_filename(image_file)

                if target_size:
                    process_image(image_path, target_size)
                else:
                    process_image(image_path)

process_images_in_folder(images_folder)

print("All images processed.")
